@@ -27,7 +27,7 @@
 
 static inline void skb_clear_rxhash(struct sk_buff *skb)
 {
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,35)
+#ifdef HAVE_RXHASH
 	skb->rxhash = 0;
 #endif
 #ifdef HAVE_L4_RXHASH
