@@ -23,6 +23,8 @@
 
 struct flow;
 struct ofpbuf;
+struct netdev;
+struct flow_wildcards;
 
 #define CFM_RANDOM_VLAN UINT16_MAX
 
@@ -32,8 +34,7 @@ struct ofpbuf;
     CFM_FAULT_REASON(MAID, maid)           \
     CFM_FAULT_REASON(LOOPBACK, loopback)   \
     CFM_FAULT_REASON(OVERFLOW, overflow)   \
-    CFM_FAULT_REASON(OVERRIDE, override)   \
-    CFM_FAULT_REASON(INTERVAL, interval)
+    CFM_FAULT_REASON(OVERRIDE, override)
 
 enum cfm_fault_bit_index {
 #define CFM_FAULT_REASON(NAME, STR) CFM_FAULT_INDEX_##NAME,
@@ -53,6 +54,7 @@ struct cfm_settings {
     uint64_t mpid;              /* The MPID of this CFM. */
     int interval;               /* The requested transmission interval. */
     bool extended;              /* Run in extended mode. */
+    bool demand;                /* Run in demand mode. */
     bool opup;                  /* Operational State. */
     uint16_t ccm_vlan;          /* CCM Vlan tag. Zero if none.
                                    CFM_RANDOM_VLAN if random. */
@@ -62,20 +64,24 @@ struct cfm_settings {
 };
 
 void cfm_init(void);
-struct cfm *cfm_create(const char *name);
-void cfm_destroy(struct cfm *);
+struct cfm *cfm_create(const struct netdev *);
+struct cfm *cfm_ref(const struct cfm *);
+void cfm_unref(struct cfm *);
 void cfm_run(struct cfm *);
 bool cfm_should_send_ccm(struct cfm *);
 void cfm_compose_ccm(struct cfm *, struct ofpbuf *packet, uint8_t eth_src[6]);
 void cfm_wait(struct cfm *);
 bool cfm_configure(struct cfm *, const struct cfm_settings *);
-bool cfm_should_process_flow(const struct cfm *cfm, const struct flow *);
+void cfm_set_netdev(struct cfm *, const struct netdev *);
+bool cfm_should_process_flow(const struct cfm *cfm, const struct flow *,
+                             struct flow_wildcards *);
 void cfm_process_heartbeat(struct cfm *, const struct ofpbuf *packet);
 int cfm_get_fault(const struct cfm *);
+uint64_t cfm_get_flap_count(const struct cfm *);
 int cfm_get_health(const struct cfm *);
 int cfm_get_opup(const struct cfm *);
-void cfm_get_remote_mpids(const struct cfm *, const uint64_t **rmps,
-                          size_t *n_rmps);
+void cfm_get_remote_mpids(const struct cfm *, uint64_t **rmps, size_t *n_rmps);
 const char *cfm_fault_reason_to_str(int fault);
 
+long long int cfm_wake_time(struct cfm*);
 #endif /* cfm.h */
